#include "stdafx.h"
#include "cpdh.h"


int main()
{
  //Test all of the functions in CPdh.h
  try
  {
    //CPdhCounter::ConnectMachine, Change the name of the machine
    //or comment out to skip over
    //CPdh::ConnectMachine(_T("BINKY")); //Uncomment to try out on your network

    CPdh::SetDefaultRealTimeDataSource(DATA_SOURCE_REGISTRY);

    //CPdhCounter::EnumObjects
    CPDHStringArray Objects{CPdh::EnumObjects(nullptr, nullptr, PERF_DETAIL_WIZARD, TRUE)};
    _tprintf(_T("Objects on the local machine include:\n"));
    for (const auto& object : Objects)
      _tprintf(_T("Object: %s\n"), object.c_str());

    Objects = CPdh::EnumObjectsH(nullptr, nullptr, PERF_DETAIL_WIZARD, TRUE);

    //CPdhCounter::EnumMachine
    CPDHStringArray Machines{CPdh::EnumMachines(nullptr)};
    _tprintf(_T("Machines PDH are connected to include:\n"));
    for (const auto& machine : Machines)
      _tprintf(_T("Machine: %s\n"), machine.c_str());
    Machines = CPdh::EnumMachinesH(nullptr);

    //CPdhCounter::EnumObjectItems
    try
    {
      CPDHStringArray Counters;
      CPDHStringArray Instances;
      CPdh::EnumObjectItems(nullptr, nullptr, _T("Process"), Counters, Instances, PERF_DETAIL_WIZARD);
      _tprintf(_T("Process Instances on this machine include:\n"));
      for (const auto& instance : Instances)
        _tprintf(_T("Instance: %s\n"), instance.c_str());

      _tprintf(_T("Process Counters on the local machine include:\n"));
      for (const auto& counter : Counters)
        _tprintf(_T("Counter: %s\n"), counter.c_str());

      CPdh::EnumObjectItemsH(nullptr, nullptr, _T("Processor Information"), Counters, Instances, PERF_DETAIL_WIZARD);      
    }
    catch (CPdhException& /*e*/)
    {
    }

    //CPDHStringArray LogSetNames;
    //CPdh::EnumLogSetNames(nullptr, LogSetNames);

    //CPdhCounter::ExpandPath
    CPDHStringArray Paths;
    try
    {
      Paths = CPdh::ExpandPath(_T("\\Processor(*)\\*"));
      _tprintf(_T("All of the counters related to the Processor on this machine are:\n"));
      for (const auto& path : Paths)
        _tprintf(_T("%s\n"), path.c_str());
    }
    catch (CPdhException& /*e*/)
    {
    }

    try
    {
      Paths = CPdh::ExpandWildCardPath(nullptr, _T("\\Processor(*)\\*"), 0);
      Paths = CPdh::ExpandWildCardPathH(nullptr, _T("\\Processor(*)\\*"), 0);
    }
    catch (CPdhException& /*e*/)
    {
    }

    //CPdhCounter::ValidatePath
    try
    {
      CPdh::ValidatePath(_T("\\Process(Idle)\\% Processor Time"));
      //CPdh::ValidatePath(nullptr, _T("\\Process(Idle)\\% Processor Time"));
    }
    catch (CPdhException& /*e*/)
    {
    }

    //CPdhCounter::GetDefaultCounter
    CPDHString sDefaultCounterName;
    try
    {
      CPdh::GetDefaultCounter(nullptr, _T("Processor"), sDefaultCounterName);
      _tprintf(_T("Default counter for Processor is %s\n"), sDefaultCounterName.c_str());
    }
    catch (CPdhException& /*e*/)
    {
    }

    //CPdhCounter::GetDefaultObject
    CPDHString sDefaultObjectName;
    try
    {
      CPdh::GetDefaultObject(nullptr, nullptr, sDefaultObjectName);
      _tprintf(_T("Default object for the local machine is %s\n"), sDefaultObjectName.c_str());
      CPdh::GetDefaultObjectH(nullptr, nullptr, sDefaultObjectName);
    }
    catch (CPdhException& /*e*/)
    {
    }

    //CPdhCounter::ParseInstanceName
    CPDHString sInstanceName;
    CPDHString sParentName;
    DWORD dwIndex{0};
    CPdh::ParseInstanceName(_T("Process/% Processor Time#3"), sInstanceName, sParentName, dwIndex);

    //CPdhCounter::BrowseCounters
    PDH_BROWSE_DLG_CONFIG BrowseDlgData{};
    BrowseDlgData.bSingleCounterPerAdd = TRUE;
    BrowseDlgData.bSingleCounterPerDialog = TRUE;
    ATL::CAtlString sCounter;
    BrowseDlgData.szReturnPathBuffer = sCounter.GetBuffer(_MAX_PATH);
    BrowseDlgData.cchReturnPathLength = _MAX_PATH;
    BrowseDlgData.dwDefaultDetailLevel = PERF_DETAIL_WIZARD;
    CPdh::BrowseCounters(BrowseDlgData);
    sCounter.ReleaseBuffer();

    PDH_BROWSE_DLG_CONFIG_H BrowseDlgDataH{};
    BrowseDlgDataH.bSingleCounterPerAdd = TRUE;
    BrowseDlgDataH.bSingleCounterPerDialog = TRUE;
    BrowseDlgDataH.szReturnPathBuffer = sCounter.GetBuffer(_MAX_PATH);
    BrowseDlgDataH.cchReturnPathLength = _MAX_PATH;
    BrowseDlgDataH.dwDefaultDetailLevel = PERF_DETAIL_WIZARD;
    CPdh::BrowseCounters(BrowseDlgDataH);
    sCounter.ReleaseBuffer();


    //Constructors
    CPdhQuery query;
    CPdhCounter counter(_T("\\Processor(_Total)\\% Processor Time"));

    //CPdhQuery::Open
    //query.Open(nullptr);
    query.OpenH(nullptr);

    //CPdhQuery::Add
#pragma warning(suppress: 26486)
    query.Add(counter);

    //CPdhQuery::Collect
    query.Collect();
    //query.Collect(1000, nullptr);
    //LONGLONG llTime = query.CollectWithTime();

    //CPdhCounter::GetInfo
    CPdhCounterInfo info;
#pragma warning(suppress: 26486)
    counter.GetInfo(info);

    //CPdhCounter::GetRawValue
    PDH_RAW_COUNTER rc1{};
    DWORD dwType{counter.GetRawValue(rc1)}; //NOLINT(clang-analyzer-deadcode.DeadStores, clang-diagnostic-unused-but-set-variable)

    //CPdhCounter::GetFormattedValue
#pragma warning(suppress: 26476)
    PDH_FMT_COUNTERVALUE fv{};
    //dwType = counter.GetFormattedValue(PDH_FMT_LONG, fv);


    //CPdhCounter::CalculateFromRawValue
    PDH_TIME_INFO pdhTimeInfo{};
    pdhTimeInfo.StartTime = 0;
    pdhTimeInfo.EndTime = 0xFFFFFFFFFFFFFFFF;
    query.SetTimeRange(&pdhTimeInfo);
    query.Collect();
    const BOOL bRealTime{query.IsRealTime()};
    UNREFERENCED_PARAMETER(bRealTime);
    PDH_RAW_COUNTER rc2{};
    query.Collect();
    dwType = counter.GetRawValue(rc2); //NOLINT(clang-analyzer-deadcode.DeadStores)
    //PDH_FMT_COUNTERVALUE fv2;
    //counter.CalculateFromRawValue(PDH_FMT_LONG, rc1, rc2, fv2);  //Uncomment to try out, Remember only
                                                                   //Some Pdh instances make sense to do this

    Sleep(1000);
    query.Collect();
    PDH_RAW_COUNTER rc3{};
    dwType = counter.GetRawValue(rc3); //NOLINT(clang-analyzer-deadcode.DeadStores)
    //PDH_FMT_COUNTERVALUE fv2;
    //CPdh::FormatFromRawValue(info.m_dwType, PDH_FMT_LONG, nullptr, &rc1, &rc3, &fv2);

    const LONGLONG nTimeBase{counter.GetTimeBase()};
    UNREFERENCED_PARAMETER(nTimeBase);

    DWORD dwItemsInArray{0};
    PPDH_FMT_COUNTERVALUE_ITEM pCounterFmtItem{counter.GetFormattedCounterArray(PDH_FMT_DOUBLE, dwItemsInArray)};
    if (pCounterFmtItem != nullptr)
      counter.DeallocateMemory(pCounterFmtItem);

    PPDH_RAW_COUNTER_ITEM pCounterRawItem{counter.GetRawCounterArray(dwItemsInArray)};
    if (pCounterRawItem != nullptr)
      counter.DeallocateMemory(pCounterRawItem);

    //SetScaleFactor
    counter.SetScaleFactor(PDH_MIN_SCALE);
#pragma warning(suppress: 26486)
    dwType = counter.GetFormattedValue(PDH_FMT_LONG, fv); //NOLINT(clang-analyzer-deadcode.DeadStores)

    //CPdhCounter::SplitPath
    CPDHString sMachineName;
    CPDHString sObjectName;
    CPDHString sParentInstance;
    DWORD dwInstanceIndex{0};
    CPDHString sCounterName;
    counter.SplitPath(sMachineName, sObjectName, sInstanceName, sParentInstance, dwInstanceIndex, sCounterName);

    //The second form of the CPdhCounter constructor
    PDH_COUNTER_PATH_ELEMENTS CounterPathElements{};
    ATL::CAtlString sTempMachineName{_T("\\\\NEPTUNE")};
    CounterPathElements.szMachineName = sTempMachineName.GetBuffer();
    ATL::CAtlString sTempObjectName{_T("System")};
    CounterPathElements.szObjectName = sTempObjectName.GetBuffer();
#pragma warning(suppress: 26472)
    CounterPathElements.dwInstanceIndex = static_cast<DWORD>(-1);
    ATL::CAtlString sTempCounterName{_T("System Up Time")};
    CounterPathElements.szCounterName = sTempCounterName.GetBuffer();
    CPdhCounter counter2{CounterPathElements};
    sTempCounterName.ReleaseBuffer();
    sTempObjectName.ReleaseBuffer();
    sTempMachineName.ReleaseBuffer();

    DWORD dwNumEntries{0};
    std::vector<PDH_TIME_INFO> timeInfo{2, std::allocator<PDH_TIME_INFO>{}};
#pragma warning(suppress: 26472)
    auto dwBufSize{static_cast<DWORD>(timeInfo.size() * sizeof(PDH_TIME_INFO))};
    try
    {
      CPdh::GetDataSourceTimeRange(nullptr, &dwNumEntries, timeInfo.data(), &dwBufSize);
    }
    catch (CPdhException& /*e*/)
    {
    }

    try
    {
      CPdh::GetDataSourceTimeRangeH(nullptr, &dwNumEntries, timeInfo.data(), &dwBufSize);
    }
    catch (CPdhException& /*e*/)
    {
    }

    CPDHString sPerfName{CPdh::LookupPerfNameByIndex(nullptr, 4)};
    try
    {
      const DWORD dwPerfIndex{CPdh::LookupPerfIndexByName(nullptr, sPerfName.c_str())};
      UNREFERENCED_PARAMETER(dwPerfIndex);
    }
    catch (CPdhException& /*e*/)
    {
    }

    CPdhDataSource dataSource;
    CPDHStringArray logFiles{_T("c:\\datasource.csv"), _T("c:\\datasource2.csv")};
    //dataSource.Open(logFiles);

    /*
    DWORD dwLogType{PDH_LOG_TYPE_CSV};
    dataSource.Open(_T("c:\\PDHDataSource.csv"), PDH_LOG_WRITE_ACCESS | PDH_LOG_CREATE_ALWAYS, &dwLogType, query, 0, nullptr);
    for (int i{0}; i<20; i++)
    {
      LONGLONG nLogFileSize{dataSource.GetSize()};
      UNREFERENCED_PARAMETER(nLogFileSize);

      _tprintf(_T("Writing value of counter to log file\n"));
      dataSource.Update(_T("This is a comment"));
      Sleep(1000);
    }
    dataSource.Close();

    CPdhDataSource dataSource2;
    CPDHStringArray logFiles2{_T("c:\\PDHDataSource.csv")};
    dataSource2.Open(logFiles2);
    FILETIME ft{};
    PPDH_RAW_LOG_RECORD pRawLogRecord = dataSource2.ReadRawRecord(ft);
    if (pRawLogRecord)
      delete [] pRawLogRecord;
    */

    CPDHString sDataSource{CPdh::SelectDataSource(GetDesktopWindow(), 0)};

    //CPdhQuery::Remove
    query.Remove(counter);
  }
#pragma warning(suppress: 26496)
  catch (CPdhException& e)
  {
#pragma warning(suppress: 26472)
    _tprintf(_T("Unexpected CPdhException occurred, Error:0x%08X\n"), static_cast<unsigned int>(e.m_Status));
  }

  return 0;
}
